@@ -8,7 +8,7 @@ from functools import partial
 from asks.sessions import Session
 
 
-__all__ = ['get', 'head', 'post', 'put', 'delete', 'options', 'request']
+__all__ = ['get', 'head', 'post', 'put', 'delete', 'options', 'patch', 'request']
 
 
 async def request(method, uri, **kwargs):
@@ -25,7 +25,8 @@ async def request(method, uri, **kwargs):
         Response (asks.Response): The Response object.
     '''
     c_interact = kwargs.pop('persist_cookies', None)
-    async with Session(persist_cookies=c_interact) as s:
+    ssl_context = kwargs.pop('ssl_context', None)
+    async with Session(persist_cookies=c_interact, ssl_context=ssl_context) as s:
         r = await s.request(method, url=uri, **kwargs)
         return r
 
@@ -37,3 +38,4 @@ post = partial(request, 'POST')
 put = partial(request, 'PUT')
 delete = partial(request, 'DELETE')
 options = partial(request, 'OPTIONS')
+patch = partial(request, 'PATCH')
