@@ -25,13 +25,17 @@ class BadHttpResponse(AsksException):
     pass
 
 
+class BadStatus(AsksException):
+    def __init__(self, err, response, status_code=500):
+        super().__init__(err)
+        self.response = response
+        self.status_code = status_code
+    pass
+
+
 class RequestTimeout(ConnectivityError):
     pass
 
 
 class ServerClosedConnectionError(ConnectivityError):
     pass
-
-
-class StreamInProgressError(AsksException):
-    pass
