@@ -5,35 +5,33 @@ for sending them, as well as receiving responses.
 This is the oldest part of asks, and as such is currently not the cleanest
 it could be. Refactors are required to bring it up to spec!
 '''
-
-__all__ = ['Request']
+__all__ = ['RequestProcessor']
 
 
 from numbers import Number
 from os.path import basename
-from urllib.parse import urlparse, urlunparse, quote_plus
+from urllib.parse import urljoin, urlparse, urlunparse, quote_plus
 import json as _json
 from random import randint
 import mimetypes
 import re
 
+from anyio import aopen
 import h11
-from h11 import RemoteProtocolError
-from multio import asynclib
 
 from .utils import requote_uri
+from .cookie_utils import parse_cookies
 from .auth import PreResponseAuth, PostResponseAuth
 from .req_structs import CaseInsensitiveDict as c_i_dict
-from .response_objects import Response, StreamBody
+from .response_objects import Response, StreamResponse, StreamBody
 from .errors import TooManyRedirects
 
 
 _BOUNDARY = "8banana133744910kmmr13a56!102!" + str(randint(1e3, 9e3))
 _WWX_MATCH = re.compile(r'\Aww.\.')
-_MAX_BYTES = 4096
 
 
-class Request:
+class RequestProcessor:
     '''
     Handles the building, formatting and i/o of requests once the calling
     session passes the required info and calls `make_request`.
@@ -67,6 +65,8 @@ class Request:
         callback (func): A callback function to be called on each bytechunk of
             of the response body.
 
+        stream (bool): Whether or not to return a StreamResponse vs Response
+
         timeout (int or float): A numeric representation of the longest time to
             wait on a complete response once a request has been sent.
 
@@ -87,6 +87,7 @@ class Request:
         self.port = port
         self.auth = None
         self.auth_off_domain = None
+        self.body = None
         self.data = None
         self.params = None
         self.headers = None
@@ -134,7 +135,7 @@ class Request:
                 redirects, the redirect responses will be stored in the final
                 response object's `.history`.
         '''
-        hconnection = h11.Connection(our_role=h11.CLIENT)
+        h11_connection = h11.Connection(our_role=h11.CLIENT)
         (self.scheme,
             self.host,
             self.path,
@@ -151,17 +152,19 @@ class Request:
         host = (self.host if (self.port == '80' or
                               self.port == '443')
                 else self.host.split(':')[0] + ':' + self.port)
+
         # default header construction
         asks_headers = c_i_dict([('Host', host),
-                                 ('Connection', 'keep-alive'),
+                                 ('Connection', 'close'),
                                  ('Accept-Encoding', 'gzip, deflate'),
                                  ('Accept', '*/*'),
                                  ('Content-Length', '0'),
-                                 ('User-Agent', 'python-asks/1.5.19')
+                                 ('User-Agent', 'python-asks/2.3.7')
                                  ])
 
         # check for a CookieTracker object, and if it's there inject
         # the relevant cookies in to the (next) request.
+        # What the fuck is this shit.
         if self.persist_cookies is not None:
             self.cookies.update(
                 self.persist_cookies.get_additional_cookies(
@@ -172,10 +175,11 @@ class Request:
 
         # handle building the request body, if any
         body = ''
-        if any((self.data, self.files, self.json)):
+        if any((self.data, self.files, self.json is not None)):
             content_type, content_len, body = await self._formulate_body()
             asks_headers['Content-Type'] = content_type
             asks_headers['Content-Length'] = content_len
+            self.body = body
 
         # add custom headers, if any
         # note that custom headers take precedence
@@ -209,7 +213,7 @@ class Request:
                           headers=asks_headers.items())
 
         # call i/o handling func
-        response_obj = await self._request_io(req, req_body, hconnection)
+        response_obj = await self._request_io(req, req_body, h11_connection)
 
         # check to see if the final socket object is suitable to be returned
         # to the calling session's connection pool.
@@ -225,7 +229,7 @@ class Request:
 
         return self.sock, response_obj
 
-    async def _request_io(self, h11_request, h11_body, hconnection):
+    async def _request_io(self, h11_request, h11_body, h11_connection):
         '''
         Takes care of the i/o side of the request once it's been built,
         and calls a couple of cleanup functions to check for redirects / store
@@ -235,7 +239,7 @@ class Request:
             h11_request (h11.Request): A h11.Request object
             h11_body (h11.Data): A h11.Data object, representing the request
                                  body.
-            hconnection (h11.Connection): The h11 connection for the request.
+            h11_connection (h11.Connection): The h11 connection for the request.
 
         Returns:
             (Response): The final response object, including any response
@@ -245,9 +249,9 @@ class Request:
             This function sets off a possible call to `_redirect` which
             is semi-recursive.
         '''
-        await self._send(h11_request, h11_body, hconnection)
-        response_obj = await self._catch_response(hconnection)
-        response_obj._parse_cookies(self.host)
+        await self._send(h11_request, h11_body, h11_connection)
+        response_obj = await self._catch_response(h11_connection)
+        parse_cookies(response_obj, self.host)
 
         # If there's a cookie tracker object, store any cookies we
         # might've picked up along our travels.
@@ -334,22 +338,14 @@ class Request:
 
         if redirect:
             allow_redirect = True
-            redirect_uri = urlparse(location.strip())
-            # relative redirect
-            if not redirect_uri.netloc:
-                self.uri = urlunparse(
-                    (self.scheme, self.host, *redirect_uri[2:]))
-
-            # absolute-redirect
-            else:
-                location = location.strip()
-                if self.auth is not None:
-                    if not self.auth_off_domain:
-                        allow_redirect = self._location_auth_protect(location)
-                self.uri = location
-                l_scheme, l_netloc, *_ = urlparse(location)
-                if l_scheme != self.scheme or l_netloc != self.host:
-                    await self._get_new_sock()
+            location = urljoin(self.uri, location.strip())
+            if self.auth is not None:
+                if not self.auth_off_domain:
+                    allow_redirect = self._location_auth_protect(location)
+            self.uri = location
+            l_scheme, l_netloc, *_ = urlparse(location)
+            if l_scheme != self.scheme or l_netloc != self.host:
+                await self._get_new_sock()
 
             # follow redirect with correct http method type
             if force_get:
@@ -393,7 +389,7 @@ class Request:
         multipart_ctype = 'multipart/form-data; boundary={}'.format(_BOUNDARY)
 
         if self.data is not None:
-            if self.files or self.json:
+            if self.files or self.json is not None:
                 raise TypeError('data arg cannot be used in conjunction with'
                                 'files or json arg.')
             c_type = 'application/x-www-form-urlencoded'
@@ -404,7 +400,7 @@ class Request:
                 c_type = self.mimetype or 'text/plain'
 
         elif self.files is not None:
-            if self.data or self.json:
+            if self.data or self.json is not None:
                 raise TypeError('files arg cannot be used in conjunction with'
                                 'data or json arg.')
             c_type = multipart_ctype
@@ -419,7 +415,8 @@ class Request:
 
         return c_type, str(len(body)), body
 
-    def _dict_to_query(self, data, params=True, base_query=False):
+    @staticmethod
+    def _dict_to_query(data, params=True, base_query=False):
         '''
         Turns python dicts in to valid body-queries or queries for use directly
         in the request url. Unlike the stdlib quote() and it's variations,
@@ -434,7 +431,7 @@ class Request:
         query = []
 
         for k, v in data.items():
-            if not v:
+            if v is None:
                 continue
             if isinstance(v, (str, Number)):
                 query.append('='.join(quote_plus(x) for x in (k, str(v))))
@@ -504,14 +501,10 @@ class Request:
         return multip_pkg
 
     async def _file_manager(self, path):
-        try:
-            async with asynclib.aopen(path, 'rb') as f:
-                return b''.join(await f.readlines()) + b'\r\n'
-        except AttributeError:
-            async with await asynclib.aopen(path, 'rb') as f:
-                return b''.join(await f.readlines()) + b'\r\n'
+        async with await aopen(path, 'rb') as f:
+            return b''.join(await f.readlines()) + b'\r\n'
 
-    async def _catch_response(self, hconnection):
+    async def _catch_response(self, h11_connection):
         '''
         Instantiates the parser which manages incoming data, first getting
         the headers, storing cookies, and then parsing the response's body,
@@ -530,7 +523,7 @@ class Request:
             The most recent response object.
         '''
 
-        response = await self._recv_event(hconnection)
+        response = await self._recv_event(h11_connection)
 
         resp_data = {'encoding': self.encoding,
                      'method': self.method,
@@ -545,7 +538,7 @@ class Request:
                      }
 
         for header in response.headers:
-            if header[0] == b'set-cookie':
+            if header[0].lower() == b'set-cookie':
                 try:
                     resp_data['headers']['set-cookie'].append(str(header[1],
                                                                   'utf-8'))
@@ -569,42 +562,50 @@ class Request:
 
         if get_body:
             if self.callback is not None:
-                endof = await self._body_callback(hconnection)
-            elif self.stream is not None:
-                if 199 < resp_data['status_code'] < 300:
-                    if not ((self.scheme == self.initial_scheme and
-                            self.host == self.initial_netloc) or
-                            resp_data['headers']['connection'].lower() == 'close'):
-                        self.sock._active = False
-                    resp_data['body'] = StreamBody(
-                        hconnection,
-                        self.sock,
-                        resp_data['headers'].get('content-encoding', None),
-                        resp_data['encoding'])
-                    self.streaming = True
+                endof = await self._body_callback(h11_connection)
+
+            elif self.stream:
+                if not ((self.scheme == self.initial_scheme and
+                        self.host == self.initial_netloc) or
+                        resp_data['headers']['connection'].lower() == 'close'):
+                    self.sock._active = False
+
+                resp_data['body'] = StreamBody(
+                    h11_connection,
+                    self.sock,
+                    resp_data['headers'].get('content-encoding', None),
+                    resp_data['encoding'])
+
+                self.streaming = True
+
             else:
                 while True:
-                    data = await self._recv_event(hconnection)
+                    data = await self._recv_event(h11_connection)
+
                     if isinstance(data, h11.Data):
                         resp_data['body'] += data.data
+
                     elif isinstance(data, h11.EndOfMessage):
                         break
+
         else:
-            endof = await self._recv_event(hconnection)
+            endof = await self._recv_event(h11_connection)
             assert isinstance(endof, h11.EndOfMessage)
 
+        if self.streaming:
+            return StreamResponse(**resp_data)
+
         return Response(**resp_data)
 
-    async def _recv_event(self, hconnection):
+    async def _recv_event(self, h11_connection):
         while True:
-            event = hconnection.next_event()
+            event = h11_connection.next_event()
             if event is h11.NEED_DATA:
-                hconnection.receive_data(
-                    (await asynclib.recv(self.sock, 10000)))
+                h11_connection.receive_data(await self.sock.receive_some(10000))
                 continue
             return event
 
-    async def _send(self, request_bytes, body_bytes, hconnection):
+    async def _send(self, request_bytes, body_bytes, h11_connection):
         '''
         Takes a package and body, combines then, then shoots 'em off in to
         the ether.
@@ -613,11 +614,10 @@ class Request:
             package (list of str): The header package.
             body (str): The str representation of the body.
         '''
-        await asynclib.sendall(self.sock, hconnection.send(request_bytes))
+        await self.sock.send_all(h11_connection.send(request_bytes))
         if body_bytes is not None:
-            await asynclib.sendall(self.sock, hconnection.send(body_bytes))
-        await asynclib.sendall(
-            self.sock, hconnection.send(h11.EndOfMessage()))
+            await self.sock.send_all(h11_connection.send(body_bytes))
+        await self.sock.send_all(h11_connection.send(h11.EndOfMessage()))
 
     async def _auth_handler_pre(self):
         '''
@@ -661,7 +661,7 @@ class Request:
             if response_obj.status_code == 401:
                 if not self.auth.auth_attempted:
                     self.history_objects.append(response_obj)
-                    r = await self.make_request()
+                    _, r = await self.make_request()
                     self.auth.auth_attempted = False
                     return r
                 else:
@@ -699,14 +699,14 @@ class Request:
             else:
                 return True
 
-    async def _body_callback(self, hconnection):
+    async def _body_callback(self, h11_connection):
         '''
         A callback func to be supplied if the user wants to do something
         directly with the response body's stream.
         '''
         # pylint: disable=not-callable
         while True:
-            next_event = await self._recv_event(hconnection)
+            next_event = await self._recv_event(h11_connection)
             if isinstance(next_event, h11.Data):
                 await self.callback(next_event.data)
             else:
